const express = require("express");
const mongoose = require("mongoose");
const axios = require("axios").default;
//import {cronPlugin} from 'mongoose-cron';
const cron = require("node-cron");
const bodyParser = require("body-parser");
const axiosHttps = require("axios-https-proxy-fix");
const fs = require("fs");
const https = require("https");

require("dotenv-safe").config();
var jwt = require("jsonwebtoken");

require("./models/Payment");
const Payment = mongoose.model("payment");

const app = express();

app.use(bodyParser.text());
app.use(function (req, res, next) {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "*");
  res.header("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE");
  res.header("Access-Control-Allow-Credentials", true);
  res.header("Access-Control-Max-Age", "86400");
  next();
});

const Sentry = require("@sentry/node");
// or use es6 import statements
// import * as Sentry from '@sentry/node';

const Tracing = require("@sentry/tracing");
// or use es6 import statements
// import * as Tracing from '@sentry/tracing';

Sentry.init({
  dsn: "https://46cb18bdc5f94089a7756dc5e1d48b37@o763905.ingest.sentry.io/5794522",

  // Set tracesSampleRate to 1.0 to capture 100%
  // of transactions for performance monitoring.
  // We recommend adjusting this value in production
  tracesSampleRate: 1.0,
});



mongoose
  .connect("mongodb://root:Cayc1818*@167.71.155.75:27017/?authSource=admin", {
    useNewUrlParser: true,
    useUnifiedTopology: true,
    useFindAndModify: false,
    useCreateIndex: true,
  })
  .then(() => {
    console.log("Conexão com MongoDB realizada com sucesso!");
  })
  .catch((erro) => {
    Sentry.captureException(erro);
    console.log("Erro. Conexão não realizada");
  });

function verifyJWT(req, res, next) {
  var token = req.headers["x-access-token"];
  if (!token)
    return res.status(401).json({ auth: false, message: "No token provided." });

  jwt.verify(token, process.env.SECRET, function (err, decoded) {
    if (err)
      return res
        .status(500)
        .json({ auth: false, message: "Failed to authenticate token." });

    req.userId = decoded.id;
    next();
  });
}

const sleep = (milliseconds) => {
  return new Promise((resolve) => setTimeout(resolve, milliseconds));
};

const cancelPayment = (responseCharge, token_shop, hasError, res) => {
  //function cancelPayment(responseCharge,token_shop,hasError){
  let cancelCallback =
    process.env.APIPAGSEGURO +
    "/charges/" +
    responseCharge.data.charges[0].id +
    "/cancel";

  return axios
    .post(
      cancelCallback,
      { amount: { value: responseCharge.data.charges[0].amount.value } },
      {
        headers: {
          Authorization: "Bearer " + token_shop,
          "x-api-version": "4.0",
          "Content-type": "application/json",
        },
      }
    )
    .then(function (response) {
      console.log("cancelando");
      let msgInfo;

      if (response.status == 201) {
        msgInfo = "Houve um erro ao realizar o pedido, tente novamente";
        Sentry.captureMessage("Houve um erro ao realizar o pedido, tente novamente==200");

      
      }

      if (response.status !== 201) {
        msgInfo = "Houve um erro ao cancelar o pedido, tente novamente";
        Sentry.captureMessage("Houve um erro ao realizar o pedido, tente novamente!200");
      }

      return msgInfo;
    })
    
    ;
};

//authentication
app.post("/login", (req, res) => {
  console.log("eita");

  const bodyParsed = JSON.parse(req.body);
  console.log(JSON.parse(req.body));

  if (bodyParsed.user === "1900" && bodyParsed.pwd === "KLJDPA&#jd") {
    //auth ok
    const id = 1; //esse id viria do banco de dados
    var token = jwt.sign({ id }, process.env.SECRET, {
      expiresIn: 300000, // expires in 5min
    });
    return res.json({ auth: true, token: token });
  }

  res.status(500).json({ message: "Login inválido!" });
});

app.post("/logout", function (req, res) {
  res.json({ auth: false, token: null });
});

app.post("/public-key", function (req, res) {
  const bodyParsed = JSON.parse(req.body);
  let hasError = false;
  let public_key_pagseguro;

  const sendGetRequest = async (res) => {
    try {
      const response = await axios.post(
        process.env.APIPAGSEGURO + "/public-keys",
        {
          type: "card",
        },
        {
          headers: { Authorization: "Bearer " + bodyParsed.token_shop },
        }
      );

      public_key_pagseguro = response.data.public_key;

      console.log(public_key_pagseguro);
    } catch (err) {
      console.log("caiu no catch");
      if (err.response.status !== 200 && err.response.data.error_messages) {
        console.log("entro");
        hasError = {
          error: true,
          message: err.response.data.error_messages[0],
          details: "Pagseguro: " + err.response.statusText,
          id: payment._id,
        };
        console.log(hasError);
      }
    }

    console.log("resposta jason");
    console.log("temos error");

    if (hasError) {
      return res.status(200).json(hasError);
    } else {
      if (public_key_pagseguro) {
        return res.status(200).json({
          public_key: public_key_pagseguro,
        });
      } else {
        return res.status(200).json({
          error: true,
          message: "Chamada não é válida",
        });
      }
    }
  };

  sendGetRequest(res);

  //res.json({ auth: false, token: null });
});


app.get("/payment/:id", verifyJWT,  (req, res) => {
  query = {_id: req.params.id}



  Payment.find(query, function(err, payment){
      //console.log(payment[0].payment_method.card);
    

      return res.status(200).json([{
          amount: payment[0].amount,
          status_delivery: payment[0].status_delivery,
          status_pagseguro: 'AUTHORIZED',
          token_shop: payment[0].token_shop,
          return_shop: payment[0].return_shop
      }]);
  });
});


app.post("/payment", verifyJWT, (req, res) => {
  const bodyParsed = JSON.parse(req.body);

  const payment = Payment.create(bodyParsed, (err, payment) => {
    //console.log(payment);
    const token_shop = bodyParsed.token_shop;
    const pagseguro_data = bodyParsed;
    let public_key_pagseguro;
    let hasError = false;
    let responseChargeData;
    const retornoMsg = false;

    const sendGetRequest = async (res) => {
      try {
        const response = await axios.post(
          process.env.APIPAGSEGURO + "/public-keys",
          {
            type: "card",
          },
          {
            headers: { Authorization: "Bearer " + token_shop },
          }
        );

        public_key_pagseguro = response.data.public_key;

        var incluiP = pagseguro_data.callback.split("|");
        console.log(incluiP);

        const data = {
          reference_id: pagseguro_data.reference_id,
          customer: {
            name: incluiP[43].replace(/%20/g, " "),
            email: incluiP[55],
            tax_id: incluiP[50],
            phones: [
              {
                country: "+55",
                area: incluiP[48],
                number: incluiP[49],
                type: "MOBILE",
              },
            ],
          },
          items: [
            {
              reference_id: pagseguro_data.reference_id,
              name: "Pedido 1900",
              quantity: 1,
              unit_amount: pagseguro_data.amount.value,
              dimensions: {
                length: 12,
                width: 34,
                height: 32,
              },
              weight: 100,
            },
          ],
          qr_codes: [
            {
              amount: {
                value: pagseguro_data.amount.value,
              },
            },
          ],
          shipping: {
            address: {
              street: incluiP[44].replace(/%20/g, " "),
              number: incluiP[45],
              complement: incluiP[53].replace(/%20/g, " "),
              locality: incluiP[43].replace(/%20/g, " "),
              city: incluiP[51].replace(/%20/g, " "),
              region: incluiP[54],
              region_code: incluiP[54],
              country: "BRA",
              postal_code: incluiP[52],
            },
          },

          //description: pagseguro_data.description,
          //amount: pagseguro_data.amount,
          //payment_method: pagseguro_data.payment_method,
          //notification_urls: pagseguro_data.notification_urls
          notification_urls: [
            process.env.APIPAGSEGURO + "/payment/" + payment._id,
          ],
          charges: [
            {
              reference_id: pagseguro_data.reference_id,
              description: pagseguro_data.description,
              amount: pagseguro_data.amount,
              payment_method: pagseguro_data.payment_method,
              metadata: {
                Key: "value",
              },
              notification_urls: [
                process.env.APIPAGSEGURO + "/payment/" + payment._id,
              ],
            },
          ],
        };

        // console.log(public_key_pagseguro);

        console.log(data);

        data.charges[0].payment_method.capture = true;

        const responseCharge = await axios.post(
          process.env.APIPAGSEGURO + "/orders",
          JSON.stringify(data),
          {
            headers: {
              Authorization: "Bearer " + token_shop,
              "Content-type": "application/json",
            },
          }
        );

        console.log(responseCharge);
        console.log("data");
        console.log(responseCharge.data.charges[0]);

        if (responseCharge.data.charges[0].status == "DECLINED") {
          hasError = {
            error: true,
            message: "Erro no pagamento. Contate a operadora do seu cartão.",
          };
        }

        query = { _id: payment._id };
        Payment.updateOne(
          query,
          {
            status_pagseguro: responseCharge.data.charges[0].status,
            reference_id: responseCharge.data.charges[0].id,
            notification_urls: [
              process.env.NOTIFICATION_URL + "/" + payment._id,
            ],
            links: responseCharge.data.charges[0].links,
          },
          function (err, res) {
            // console.log(res);
          }
        );

        if (responseCharge.data.charges[0].status == "PAID") {
          responseChargeData = responseCharge;

          await sleep(1000);

          await axios
            .post(
              payment.callback +
                "&PagSeguroID=" +
                responseCharge.data.charges[0].id
            )
            .then(function (response) {
              console.log("fez chamada da central 1900");

              query = { _id: payment._id };
              let responseData;

              responseData = req.body;

              Payment.updateOne(
                query,
                {
                  status_delivery: response.data.resultado,
                  description: response.data.resultado,
                  return_shop: response.data,
                  description: response.data.descr,
                },
                function (err, res) {
                  responseData = response.data;
                }
              );

              console.log("responseeee");
              console.log(response.status);
              console.log(response.data.resultado);

              if (response.data.resultado !== "OK" || response.status !== 200) {
                const postCancel = async (
                  responseCharge,
                  token_shop,
                  hasError,
                  res
                ) => {
                  return await cancelPayment(
                    responseCharge,
                    token_shop,
                    hasError
                  );
                };

                let returnCancel = postCancel(
                  responseCharge,
                  token_shop,
                  hasError
                );

                returnCancel.then(function (result) {
                  console.log("entrando");

                  if (
                    result ==
                    "Houve um erro ao realizar o pedido, tente novamente"
                  ) {
                    Payment.updateOne(
                      query,
                      { status_pagseguro: "CANCELED" },
                      function (err, res) {}
                    );
                  } else {
                    Payment.updateOne(
                      query,
                      { status_pagseguro: "NOT-CANCELLED" },
                      function (err, res) {}
                    );
                  }

                  hasError = {
                    error: true,
                    message: result
                      ? result
                      : "Erro no pedido. Seu pagamento foi cancelado.",
                  };
                });
              }
            })
            .catch(function (error) {
              Sentry.captureMessage("Catch na central 1900");
              Sentry.captureException(error);
              console.log("vai cancelar erro na central 1900");
              console.log(responseCharge);
              console.log("query");
              console.log(payment._id);

              query = { _id: payment._id };

              const postCancel = async (
                responseCharge,
                token_shop,
                hasError,
                res
              ) => {
                return await cancelPayment(
                  responseCharge,
                  token_shop,
                  hasError
                );
              };

              const returnCancelError = postCancel(
                responseCharge,
                token_shop,
                hasError
              );

              console.log("postcancel");
              console.log(returnCancelError);


              returnCancelError.then(function (result) {
                console.log("entrando vai cancelar");

                if (
                  result ==
                  "Houve um erro ao realizar o pedido, tente novamente"
                ) {
                  Payment.updateOne(
                    query,
                    { status_pagseguro: "CANCELED" },
                    function (err, res) {}
                  );
                } else {
                  Payment.updateOne(
                    query,
                    { status_pagseguro: "NOT-CANCELLED" },
                    function (err, res) {}
                  );
                }

                hasError = {
                  error: true,
                  message: result
                    ? result
                    : "Erro no pedido. Seu pagamento foi cancelado.",
                };
              });
            });

            //MENSAGEM SUCESSO
        }

        if (responseCharge.data.charges[0].status !== "PAID") {
          const postCancel = async (
            responseCharge,
            token_shop,
            hasError,
            res
          ) => {
            return await cancelPayment(
              responseCharge,
              token_shop,
              hasError
            );
          };

          const returnCancelError = postCancel(
            responseCharge,
            token_shop,
            hasError
          );

          returnCancelError.then(function (result) {
            console.log("entrando");

            if (
              result ==
              "Houve um erro ao realizar o pedido, tente novamente"
            ) {
              Payment.updateOne(
                query,
                { status_pagseguro: "CANCELED" },
                function (err, res) {}
              );
            } else {
              Payment.updateOne(
                query,
                { status_pagseguro: "NOT-CANCELLED" },
                function (err, res) {}
              );
            }

            hasError = {
              error: true,
              message: result
                ? result
                : "Erro no pedido. Seu pagamento foi cancelado.",
            };
          });
        }  

     

      } catch (err) {
        Sentry.captureException(err);
        console.log("erro ao tentar public key pagseguro");
        //if (err.response.status !== 200 && err.response.data.error_messages) {
        

          if(payment){
            hasError = {
              error: true,
              message: 'Ops, um erro aconteceu, tente novamente.',
             // details: "Pagseguro: " + err.response.statusText,
              id: payment._id,
            };
          } else {
            hasError = {
              error: true,
              message: 'Ops, um erro aconteceu no seu id, tente novamente.'
             // details: "Pagseguro: " + err.response.statusText,
            };
          }
        //}
        //arrumar aqui o erro
      }

      await sleep(11000);
      if (hasError) {
        return res.status(200).json(hasError);
      } else {
        if (payment) {
          return res.status(200).json({
            error: false,
            message: "Requisição Pagamento realizado!",
            id: payment._id,
          });
        } else {
          return res.status(200).json({
            error: true,
            message:
              "Faltam parametros para concluir a transação, verifique o schema.",
          });
        }
      }
    };

    sendGetRequest(res);

    /////

    // PEGA PUBLIC KEY DA PAGSEGURO
    //      axios.post(process.env.APIPAGSEGURO+'/public-keys', {
    //         "type": "card"
    //     },{
    //         headers:  { Authorization: "Bearer " + token_shop }
    //     })
    //     .then(function (response) {

    //         public_key_pagseguro = response.data.public_key;
    //         const data = {

    //             reference_id: pagseguro_data.reference_id,
    //             description: pagseguro_data.description,
    //             amount: pagseguro_data.amount,
    //             payment_method: pagseguro_data.payment_method,
    //             notification_urls: pagseguro_data.notification_urls

    //         }

    //         console.log(public_key_pagseguro);

    //           axios.post(process.env.APIPAGSEGURO+'/charges', data,{
    //             headers:  { Authorization: "Bearer " + token_shop, "x-api-version":"4.0", "Content-type": "application/json" }
    //         })
    //         .then(function (response) {
    //             query = {_id: payment._id}
    //             Payment.updateOne(query, {status:response.data.status,reference_id:response.data.id, notification_urls:[process.env.NOTIFICATION_URL+'/'+payment._id], links:response.data.links }, function(err, res){
    //                // console.log(res);
    //             });

    //             console.log('aqui'+response.status);

    //         })
    //         .catch(function (error) {
    //           // console.log(error);
    //            if(error.response.status !== 200) {
    //                console.log('entro');
    //             hasError = {
    //                 error: true,
    //                 message: error.response.data.error_messages[0],
    //                 details: "Pagseguro: "+error.response.statusText,
    //             }
    //             console.log(hasError);
    //         }

    //         });

    //     })
    //     .catch(function (error) {
    //         console.log(hasError);
    //     });

    // if(err) return res.status(400).json({
    //     error: true,
    //     message: "Erro: Pagamento não realizado."
    // });

    // console.log('tem erro');
    // console.log(hasError);

    // if(!hasError){
    //     return res.status(200).json({
    //         error: false,
    //         message: "Requisição Pagamento realizado!",
    //         id:payment._id
    //     })
    // } else {
    //     return res.status(400).json(hasError);
    // }
  });
});

app.listen(8081, () => {
  console.log("Servidor iniciado na porta 8081: http://localhost:8081");
});

//CRON
// cron.schedule("*/30000 * * * * *", function() {

//     query = {$or: [
//         {status: {$ne: 'PAID'}},
//         {status: {$ne: 'COMPLETED'}},
//         {status: {$ne: 'CANCELLED'}},
//       ]}
//     Payment.find(query, function(err, res){
//         console.log(res);
//         //check if is longer than 30 seg to cancel
//         let date = new Date();
//         let SECONDS_30 = 30*1000;

//         res.forEach(function (arrayItem) {
//             let total = new Date()-arrayItem.updatedAt;
//             console.log(total);

//             if(total > SECONDS_30) {

//                 console.log('passou de 30seg, cancelando');
//                 if(arrayItem.links[1].href){
//                     axios.post(arrayItem.links[1].href, arrayItem.amount ,{
//                         headers:  { Authorization: "Bearer " + arrayItem.token_shop, "x-api-version":"4.0", "Content-type": "application/json" }
//                     })
//                     .then(function (response) {
//                         query = {_id: arrayItem._id}
//                         Payment.updateOne(query, {status:'CANCELLED' }, function(err, res){
//                         console.log('PAGAMENTO CANCELADO');
//                         });

//                     })
//                     .catch(function (error) {
//                         console.log('erro no cancelamento, não foi possivel');
//                     });
//                 }

//             } else {
//                 console.log('ñ passou de 30seg');
//             }
//         });

//     });

//     console.log("cancelled cron");
// })
